unit XStrTool;

{--------------------------------------------------------------------------}
{  Unit/Unit XStrTool                                                     }
{  Auteur/Author : Jean-Michel BOUCTON, et exemple en provenance de        }
{                  Compuserve / and example from compuserve (IsMatch,      }
{                  Matche, match_after_star)                               }
{  Cration/Create : 28/01/1997                                            }
{                                                                          }
{  Dernire modification/Last update : 28/05/1998                          }
{                                                                          }
{  Librairie de fonctions de manipulation de chaines de caractres.        }
{  Strings management library.                                             }
{--------------------------------------------------------------------------}

interface

uses
  SysUtils, Classes;

  function ExtractDelimitedStr( S : string ; delimiter : char ):string;
  function IsNumber(Ch: Char): Boolean;
  function RTrim(S: string): string;
  function LTrim(S: string): string;
  function StripFrontChars(S: string; Ch: Char): string;
  function AllTrim(S: string): string;
  function sBreakApart( BaseString
                       ,BreakString: string;
                        StringList: TStrings
                      ): TStrings;
  function pBreakApart( BaseString
                       ,BreakString: PChar;
                        StringList: TStrings
                      ): TStrings;
  function ReplaceStr(BaseString, ReplaceThis, WithThis: string): string;
  function ReplaceLStr(ToString: string;
                       FirstByte,
                       NumOfBytes: integer;
                       FromString: string
                      ): string;
  function IsMatch( pattern, text: string ): Boolean;
  function Matche( pattern, text: string ): Integer;
  function match_after_star( pattern, text: String ): Integer;
  function IsUpper(ch: char): boolean;
  function IsLower(ch: char): boolean;
  function ToUpper(ch: char): char;
  function ToLower(ch: char): char;
  function Proper(const s: string): string;
  function Occurs(SearchStr, Str : string):integer;
  function ReplaceChars(S: string; OldCh, NewCh: Char): string;
  function GetHexWord(w: Word): string;
  function PADC(InStr: string; TotLen: Byte; Ch: Char): string;
  function PADL(InStr: string; TotLen: Byte; Ch: Char): string;
  function PADR(InStr: string; TotLen: Byte; Ch: Char): string;
  function RAT(InStr: string; SearchStr: string): Byte;
  function REPLICATE(RepliStr: string; NoTimes: Byte): string;
  function RIGHT(InStr: string; Len: Byte): string;
  function AddChar(const S: string; ch: Char): string;
  function AddBackSlash(const S: string): string;
  function StripEndBackSlash(const S: string): string;
  function ExtractAllDelimitedStr( S: string;
                                   delimiter: char;
                                   StrList: TStringList
                                 ): TStringList;
  function PosChRight(Ch: Char; S: string): integer;
  function ExtractPrivDelimitedStr(S : string ; delimleft, delimright : char):string;
  function IsStrFloat(sfloat: string): boolean;
  function IsNumeric(snum: string): boolean;
  function SurroundStr(S: string; chs: string): string;
  function PrivateSurroundStr(S: string; chleft, chright: char): string;
  function StripChars(S: string; Ch: char): string;
  function IsStrDate(sdate: string): boolean;
  function IsStrTime(stime: string): boolean;
  function IsStrDateTime(sdatetime: string): boolean;
  function StripStr(SupprStr, S: string): string;
  function StrIPos(Str1, Str2: PChar):PChar;
  function StrTrim(Str: PChar): PChar;
  function Weirdcase(const S: string): string;

const
  { match defines }
  MATCH_PATTERN                 = 6;
  MATCH_LITERAL                 = 5;
  MATCH_RANGE                   = 4;
  MATCH_ABORT                   = 3;
  MATCH_END                     = 2;
  MATCH_VALID                   = 1;
  { pattern defines }
  PATTERN_VALID                 = 0;
  PATTERN_ESC                   = -1;
  PATTERN_RANGE	                = -2;
  PATTERN_CLOSE	                = -3;
  PATTERN_EMPTY                 = -4;
  { character defines }
  MATCH_CHAR_SINGLE             = '?';
  MATCH_CHAR_KLEENE_CLOSURE     = '*';
  MATCH_CHAR_RANGE_OPEN	        = '[';
  MATCH_CHAR_RANGE              = '-';
  MATCH_CHAR_RANGE_CLOSE        = ']';
  MATCH_CHAR_CARAT_NEGATE       = '^';
  MATCH_CHAR_EXCLAMATION_NEGATE	= '!';

  CHAR_TABLE: array[#0..#255] of byte =
               ( 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17,
                18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33,
                34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49,
                50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 97,
                98, 99, 100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 110,
                111, 112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122, 91,
                92, 93, 94, 95, 96, 97, 98, 99, 100, 101, 102, 103, 104, 105,
                106, 107, 108, 109, 110, 111, 112, 113, 114, 115, 116, 117, 118,
                119, 120, 121, 122, 123, 124, 125, 126, 127, 128, 129, 130, 131,
                132, 133, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143, 144,
                145, 146, 147, 148, 149, 150, 151, 152, 153, 154, 155, 156, 157,
                158, 159, 160, 161, 162, 163, 164, 165, 166, 167, 168, 169, 170,
                171, 172, 173, 174, 175, 176, 177, 178, 179, 180, 181, 182, 183,
                184, 185, 186, 187, 188, 189, 190, 191, 192, 193, 194, 195, 196,
                197, 198, 199, 200, 201, 202, 203, 204, 205, 206, 207, 208, 209,
                210, 211, 212, 213, 214, 215, 216, 217, 218, 219, 220, 221, 222,
                223, 224, 225, 226, 227, 228, 229, 230, 231, 232, 233, 234, 235,
                236, 237, 238, 239, 240, 241, 242, 243, 244, 245, 246, 247, 248,
                249, 250, 251, 252, 253, 254, 255);

implementation

function ExtractDelimitedStr( S : string ; delimiter : char ):string;
var
  str1 : string;
  idx : integer;
begin
  Result := S;
  idx := Pos ( delimiter, S );
  if ( idx > 0 ) then
  begin
    str1 := Copy( S, idx + 1, Length(S) );
    idx := Pos ( delimiter, str1 );
    if ( idx > 0 ) then
      str1 := Copy( str1, 1, idx - 1);
    Result := str1;
  end;
end;

function IsNumber(Ch: Char): Boolean;
begin
  IsNumber := ((Ch >= '0') and (Ch <= '9'));
end;

function RTrim(S: string): string;
var
  i: Integer;
begin
  i := Length(S);
  while S[i] = ' ' do begin
    Delete(S,i,1);
    Dec(i);
  end;
  Result := S;
end;

function LTrim(S: string): string;
begin
  while (S[1] = #32) and (Length(S) > 0) do begin
    Delete(S,1,1);
  end;
  Result := S;
end;

function StripFrontChars(S: string; Ch: Char): string;
begin
  while (S[1] = Ch) and (Length(S) > 0) do
    S := Copy(S,2,Length(S) - 1);
  Result := S;
end;

function AllTrim(S: string): string;
var
  Temp: String;
begin
  Temp := LTrim(S);
  Temp := RTrim(Temp);
  Result := Temp;
end;

function sBreakApart(BaseString, BreakString: string; StringList: TStrings): TStrings;
var
  EndOfCurrentString: byte;
begin
  repeat
    EndOfCurrentString := Pos(BreakString, BaseString);
    if EndOfCurrentString = 0 then
      StringList.add(BaseString)
    else
      StringList.add(Copy(BaseString, 1, EndOfCurrentString - 1));
    BaseString := Copy(  BaseString
                       , EndOfCurrentString + length(BreakString)
                       , length(BaseString) - EndOfCurrentString
                      );
  until EndOfCurrentString = 0;
  result := StringList;
end;

function ReplaceStr(BaseString, ReplaceThis, WithThis: string): string;
var
  SL: TStringList;
  i, val: integer;
begin
  SL := nil;
  val := 0;
  try
    SL := TStringList.create;
    sBreakApart(BaseString, ReplaceThis, SL);
    if SL.count > 1 then
    begin
      result := '';
      for i := 0 to SL.count - 2 do
      begin
        result := result + SL[i] + WithThis;
        val := i;
      end;
      result := result + SL[val];
    end else
      Result := BaseString;
  finally
    SL.free;
  end;
end;

function pBreakApart(BaseString, BreakString: PChar; StringList: TStrings): TStrings;
var
  BreakStringLength: word;
  pEndOfCurrentString, pEndOfBaseString: PChar;
  temp: array[0..255] of char;
begin
  BreakStringLength := StrLen(BreakString);
  pEndOfBaseString := BaseString;
  inc(pEndOfBaseString, StrLen(BaseString));
  repeat
    pEndOfCurrentString := StrPos(BaseString, BreakString);

    StringList.add(StrPas(StrLCopy(temp, BaseString, pEndOfCurrentString - BaseString)));
    inc(BaseString, pEndOfCurrentString - BaseString + BreakStringLength);
  until BaseString >= pEndOfBaseString - BreakStringLength;
  result := StringList;
end;

function ReplaceLStr( ToString: string;
                      FirstByte, NumOfBytes: integer;
                      FromString: string ): string;
begin
  delete(toString, FirstByte, NumOfBytes);
  insert(FromString, ToString, FirstByte);
  Result := ToString;
end;

{
  File: match.pas
  Author: Kevin Boylan

  This code is meant to allow wildcard pattern matches.  It is VERY useful for
  matching filename wildcard patterns. It allows unix grep-like pattern
  comparisons, for instance:

	?			Matches any single characer
	*			Matches any contiguous characters
	[abc]		Matches a or b or c at that position
	[^abc]	Matches anything but a or b or c at that position
	[!abc]	Ditto
	[a-e]		Matches a through e at that position

	'ma?ch.*'	-Would match match.exe, mavch.dat, march.on, etc
	'this [e-n]s a [!zy]est' -Would match 'this is a test', but would not
         match 'this as a yest'

  This is a Delphi VCL translation from C code that was downloaded from CIS.
  That C code was written by J. Kerceval and released to public domain 02/20/1991.
  This code is ofcourse also public domain.
  I would appreciate it if you would let me know if you find any bugs.
  I would also appreciate any notes sent my way letting me know if you find it
  useful.  My email address is

	CIS:			75221,1057
	Internet:	75221.1057@compuserve.com

}
function IsMatch( pattern, text: string ): Boolean;
begin
  Result := Matche( pattern, text ) = 1;
end;

function Matche( pattern, text: string ): Integer;
var
  range_start,
  range_end,
  p,
  t,
  plen,
  tlen	        : Integer;
  invert,
  member_match,
  loop	        : Boolean;
begin
  p := 1;
  t := 1;
  pattern := LowerCase(pattern);
  text := LowerCase(Text);
  plen := Length( pattern );
  tlen := Length( text );
  Result := 0;
  while ( (Result = 0) and (p <= plen) ) do
  begin
    if (t > tlen) then
    begin
      if (pattern[p] = MATCH_CHAR_KLEENE_CLOSURE) and (p+1 > plen) then
        Result := MATCH_VALID
      else
        Result := MATCH_ABORT;
      exit;
    end
    else
      Case (pattern[p]) of
        MATCH_CHAR_KLEENE_CLOSURE:
          Result := match_after_star( Copy(pattern,p,plen),Copy(text,t,tlen) );
        MATCH_CHAR_RANGE_OPEN:
	begin
          Inc(p);
          invert := False;
          if (pattern[p] = MATCH_CHAR_EXCLAMATION_NEGATE) or
             (pattern[p] = MATCH_CHAR_CARAT_NEGATE) then
          begin
            invert := True;
            Inc(p);
          end;
          if (pattern[p] = MATCH_CHAR_RANGE_CLOSE) then
          begin
            Result := MATCH_PATTERN;
            exit;
          end;
          member_match := False;
          loop := True;
          while ( (loop) and (pattern[p] <> MATCH_CHAR_RANGE_CLOSE) ) do
          begin
            range_start := p;
            range_end := p;
            Inc(p);
            if (p > plen) then
            begin
              Result := MATCH_PATTERN;
              exit;
            end;
            if (pattern[p] = MATCH_CHAR_RANGE) then
            begin
              Inc(p);
              range_end := p;
              if (p > plen) or (pattern[range_end] = MATCH_CHAR_RANGE_CLOSE) then
              begin
                Result := MATCH_PATTERN;
                exit;
              end;
              Inc(p);
            end;
            if (p > plen) then
            begin
              Result := MATCH_PATTERN;
              exit;
            end;
            if (range_start < range_end) then
            begin
              if (text[t] >= pattern[range_start]) and
                 (text[t] <= pattern[range_end]) then
              begin
                member_match := True;
                loop := False;
              end;
            end
            else
            begin
              if (text[t] >= pattern[range_end]) and
                 (text[t] <= pattern[range_start]) then
              begin
                member_match := True;
                loop := False;
              end;
            end; { if (range_start < range_end) }
          end; { while (loop) and ...}
          if (invert and member_match) or (not(invert or member_match)) then
          begin
            Result := MATCH_RANGE;
            exit;
          end;
          if (member_match) then
            while ((p <= plen) and (pattern[p] <> MATCH_CHAR_RANGE_CLOSE)) do
              Inc(p);
          if (p > plen) then
          begin
            Result := MATCH_PATTERN;
            exit;
          end;
        end; { MATCH_CHAR_RANGE_OPEN: }
      else
        if (pattern[p] <> MATCH_CHAR_SINGLE) then
          if (pattern[p] <> text[t]) then
            Result := MATCH_LITERAL;
      end; { Case pattern[p] }
    Inc(p);
    Inc(t);
  end; { while ( (Result := 0) and (p < plen) ) }
  if (Result = 0) then
    if (t <= tlen) then
      Result := MATCH_END
    else
      Result := MATCH_VALID;
end;

function match_after_star( pattern, text: String ): Integer;
var
  p,
  t,
  plen,
  tlen	 : Integer;
begin
  Result := 0;
  p := 1;
  t := 1;
  plen := Length(pattern);
  tlen := Length(text);
  while (( t <= tlen ) and (p < plen)) and
        (pattern[p] = MATCH_CHAR_SINGLE) or
        (pattern[p] = MATCH_CHAR_KLEENE_CLOSURE) do
  begin
    if (pattern[p] = MATCH_CHAR_SINGLE) then
      Inc(t);
    Inc(p);
  end;
  if (t > tlen) then
  begin
    Result := MATCH_ABORT;
    exit;
  end;
  if (p > plen) then
  begin
    Result := MATCH_VALID;
    exit;
  end;
  repeat
    if (pattern[p] = text[t]) or (pattern[p] = MATCH_CHAR_RANGE_OPEN) then
    begin
      pattern := Copy(pattern,p,plen);
      text := Copy(text,t,tlen);
      plen := Length(pattern);
      tlen := Length(text);
      p := 1;
      t := 1;
      Result := matche( pattern , text );
    end;
    if (t > tlen) then
    begin
      Result := MATCH_ABORT;
      exit;
    end;
    Inc(t);
  until ( Result <> 0 );
end;
{
function is_pattern( pattern: String ): Boolean;
begin
  Result := false;
end;

function is_valid_pattern( pattern: String; var error_type: Integer ): Boolean;
begin
  Result := false;
end;
}
function IsUpper(ch: char): boolean;
begin
  Result := ch in ['A'..'Z'];
end;

function IsLower(ch: char): boolean;
begin
  Result := ch in ['a'..'z'];
end;

function ToUpper(ch: char): char;
begin
  Result := chr(ord(ch) and $DF);
end;

function ToLower(ch: char): char;
begin
  Result := chr(ord(ch) or $20);
end;

function Proper(const s: string): string;
var
  i: Integer;
  CapitalizeNextLetter: Boolean;
begin
  Result := LowerCase(s);
  CapitalizeNextLetter := True;
  for i := 1 to Length(Result) do
  begin
    if CapitalizeNextLetter and IsLower(Result[i]) then
      Result[i] := ToUpper(Result[i]);
    CapitalizeNextLetter := Result[i] = ' ';
  end;
end;

function Occurs( SearchStr, Str : string ):integer;
var
  count, idx : byte;
  str1 : string;
begin
  count := 0;
  str1 := Str;
  repeat
    idx := Pos(SearchStr, str1);
    if ( idx > 0 ) then
    begin
      inc(count);
      str1 := Copy( str1, idx + Length(SearchStr), Length(str1) );
    end;
  until (idx = 0);
  Result := count;
end;

function ReplaceChars(S: string; OldCh, NewCh: Char): string;
var
  Len: Integer;
  i: Integer;
begin
  Len := Length(S);
  for i := 1 to Len do
    if S[i] = OldCh then
      S[i] := NewCh;
  Result := S;
end;

function GetHexWord(w: Word): string;
const
  HexChars: array [0..$F] of Char =  '0123456789ABCDEF';
var
  Addr: string;
begin
  Addr[1] := hexChars[Hi(w) shr 4];
  Addr[2] := hexChars[Hi(w) and $F];
  Addr[3] := hexChars[Lo(w) shr 4];
  Addr[4] := hexChars[Lo(w) and $F];
  SetLength(Addr, 4);
  Result := addr;
end;

function PADC(InStr: string; TotLen: Byte; Ch: Char): string;
var
 NoPAD  : Integer;
 TempStr: string;
Begin
  FillChar(TempStr[1], TotLen, Ch);
  SetLength(TempStr, TotLen);
  NoPad    := Length(InStr);
  If NoPAD <= TotLen then
    move(InStr[1], TempStr[((TotLen - NoPAD) div 2) + 1], NoPAD)
  else
    move(InStr[((NoPAD - TotLen) div 2) + 1],TempStr[1], TotLen);
  Result := TempStr;
End;

function PADL(InStr: string; TotLen: Byte; Ch: Char): string;
var
 TempStr  : string;
 LenInStr : Byte;
Begin
  FillChar(TempStr[1], TotLen, Ch);
  SetLength(TempStr, TotLen);
  LenInStr  := Length(InStr);
  if LenInStr <= TotLen then
    move(InStr[1], TempStr[Succ(TotLen - LenInStr)], LenInStr)
  else
    move(InStr[1], TempStr[1], TotLen);
  Result := TempStr;
End;

function PADR(InStr: string; TotLen: Byte; Ch: Char): string;
var
  TempStr  : string;
begin
  FillChar(TempStr[1], TotLen, Ch);
  SetLength(TempStr, TotLen);
  if Length(InStr) <= TotLen then
    move(InStr[1],TempStr[1],Length(InStr))
  else
    move(InStr[1], TempStr[1], TotLen);
  Result := TempStr;
end;

function RAT(InStr: string; SearchStr: string): Byte;
var
 Quit    : Boolean;
 Pos,Len : Integer;
begin
 RAT := 0;
 Quit:= False;
 Len := Length(SearchStr);
 Pos := Length(InStr) - Len;
 while Quit = False do
   begin
   if Copy(InStr,Pos,Len) = SearchStr then
   begin
     RAT := Pos;
     Quit := True;
   end;
   if Pos = 1 then
   begin
     Quit:= True;
   end;
   Dec(Pos,1);
 end;
end;

function REPLICATE(RepliStr: string; NoTimes: Byte): string;
var
 i      : Byte;
 TempStr: string;
begin
  TempStr := '';
  for i:= 1 to NoTimes do
    TempStr:= TempStr + RepliStr;
  Result := TempStr;
end;

function RIGHT(InStr: string; Len: Byte): string;
begin
  Result := Copy(InStr,((Length(InStr) - Len)+1),Length(InStr));
end;

function AddChar(const S: string; ch: Char): string;
begin
  Result := S;
  if Copy(S, Length(S), 1) <> ch then
    Result:= Result + ch;
end;

function AddBackSlash(const S: string): string;
begin
  Result := S;
  if Copy(S, Length(S), 1) <> '\'
    then Result:= S + '\';
end;

function StripEndBackSlash(const S: string): string;
begin
  Result := S;
  if Copy(S, Length(S), 1) = '\'
    then Result:= Copy(S ,1, Length(S) - 1);
end;

function ExtractAllDelimitedStr(  S: string
                                ; delimiter: char
                                ; StrList: TStringList
                               ): TStringList;
var
  str1, str2 : string;
  idx : integer;
begin
  Result := StrList;
  str1 := S;
  idx := Pos ( delimiter, str1 );
  while ( idx > 0 ) do
  begin
    str1 := Copy( str1, idx + 1, Length(str1) );
    idx := Pos ( delimiter, str1 );
    if ( idx > 0 ) then
    begin
      str2 := Copy( str1, 1, idx - 1);
      StrList.Add(str2);
      str1 := Copy( str1, idx + 1, Length(str1) );
    end;
    idx := Pos ( delimiter, str1 );
  end;
end;

function PosChRight(Ch: Char; S: string): integer;
var
  Len: Integer;
  i: Integer;
begin
  Result := 0;
  Len := Length(S);
  for i := Len downto 1 do
    if S[i] = Ch then
    begin
      Result := i;
      exit;
    end;
end;

function ExtractPrivDelimitedStr( S : string ; delimleft, delimright : char ):string;
var
  str1 : string;
  idx, idxright : integer;
begin
  Result := S;
  idx := Pos ( delimleft, S );
  idxright := PosChRight(delimright, S);
  if ( idx > 0 ) and ( idxright > 0 ) then
  begin
    str1 := Copy( S, idx + 1, (idxright - (idx+1)) );
   { idx := Pos ( delimiter, str1 );
    if ( idx > 0 ) then
      str1 := Copy( str1, 1, idx - 1); }
    Result := str1;
  end;
end;

function IsStrFloat( sfloat: string ): boolean;
begin
  try
    StrToFloat( sfloat );
  except
    Result := false;
    exit;
  end;
  Result := true;
end;

function IsNumeric( snum: string ): boolean;
begin
  try
    StrToInt( snum );
  except
    Result := false;
    exit;
  end;
  Result := true;
end;

function SurroundStr(S: string; chs: string): string;
begin
  Result := chs + S + chs;
end;

function PrivateSurroundStr(S: string; chleft, chright: char): string;
begin
  Result := chleft + S + chright;
end;

function StripChars(S: string; Ch: char): string;
var
  Len: Integer;
  i: Integer;
  stemp: string;
begin
  if Pos(Ch, S) = 0 then
  begin
    Result := S;
    exit;
  end;
  stemp := '';
  Len := Length(S);
  for i := 1 to Len do
    if S[i] <> Ch then
      stemp := stemp + S[i];
  Result := stemp;
end;

function IsStrDate(sdate: string): boolean;
begin
  try
    StrToDate( sdate );
  except
    Result := false;
    exit;
  end;
  Result := true;
end;

function IsStrTime(stime: string): boolean;
begin
  try
    StrToTime( stime );
  except
    Result := false;
    exit;
  end;
  Result := true;
end;

function IsStrDateTime(sdatetime: string): boolean;
begin
  try
    StrToDateTime( sdatetime );
  except
    Result := false;
    exit;
  end;
  Result := true;
end;

function StripStr(SupprStr, S: string): string;
var
  Len: Integer;
  ipos: Integer;
begin
  Result := S;
  if Pos(SupprStr, S) = 0 then
    exit
  else
  begin
    ipos := Pos(SupprStr, S);
    Len := Length(SupprStr);
    delete(Result, ipos, Len);
  end;
end;

function StrIPos(Str1, Str2: PChar):PChar;
var
  P, BufStr, PosPtr: PChar;
begin
  Result:= nil;
  P:= Str2;
  BufStr:= Str1;
  while BufStr^ <> #0 do
  begin
    if (CHAR_TABLE[P^] = CHAR_TABLE[BufStr^]) then
    begin
      PosPtr := BufStr;
      while P^ <> #0 do
      begin
        if (CHAR_TABLE[P^] <> CHAR_TABLE[BufStr^]) then
          break;
        inc(P);
        inc(BufStr);
      end;
      if P^ = #0 then
      begin
        Result := PosPtr;
        break;
      end;
    end else
      inc(BufStr);
    P:= Str2;
  end;
end;

function StrTrim(Str: PChar): PChar;
var
  P, PEnd: PChar;
begin
  Result := Str;
  P := Str;
  while (P^ = #32) and (StrLen(P) > 0) do
    Inc(P);
  if P > Str then
    StrCopy(Result, P);
  PEnd := Result + (StrLen(Result)- 1);
  P := PEnd;
  while (P^ = #32) and (P > Result) do
    Dec(P);
  if P < PEnd then
    StrLCopy(Result, Result, ((P - Result) + 1));
end;

function Weirdcase(const S: string): string;
var
  i: integer;
begin
  Result := LowerCase(S);
  i := 1;
  while i <= Length(S) do
  begin
    if IsLower(Result[i]) then
      Result[i] := ToUpper(Result[i]);
      inc(i, 2);
  end;
end;

end.
